import fse from 'fs-extra'
import cwd from 'cwd'
import path, {join} from 'path'
import _ from 'lodash'
import {warn, printFileStr, error} from 'zog'
import dedent from 'dedent-js'
import c from 'chalk'
import tildify from 'tildify'

////////////////////////////////////////////////////////////////////////////////

// Old settings

//const base = config.envProps.serveAssetsFromWebpackDevServer ? 'http://0.0.0.0:8081/build' : '/assets'
//const base = config.envProps.DEBUG ? 'http://172.168.0.110:8081' : '/assets'

// TODO(vjpr): This should be configurable so we can easily disable the vendor
// dll.
//   Don't use `envProps.DEBUG`. Use something like `USE_VENDOR_DLL`.
// Sync to webpackerator setting.

// Use precompiled dll in development.
//vendorBundleJs: config.envProps.DEBUG ? buildNoProxy + '/vendor.dll.js' : base + '/vendor.bundle.js',
// vendorBundleJs: base + '/vendor.bundle.js', // DEBUG

////////////////////////////////////////////////////////////////////////////////

// TODO(vjpr): Make this much more generic and bundle-aware.

export default function(opts) {

  // Config
  ////////////////////

  const {isDev, buildPath, buildDir} = opts

  // NOTE: In DEBUG mode we proxy asset requests to webpack dev server.
  // This setting will use a simple static web server.
  // See `live-static`.
  const noProxy = true

  const buildNoProxyPath = 'build-noproxy'

  // TODO(vjpr): Get from webpackerator or configurize.
  const useVendorDll = process.env.USE_VENDOR_DLL
  const env = opts.env

  ////////////////////

  // TODO(vjpr): It may happen that the user runs `npm start` without
  //   webpack-dev-server running and they have not compiled the vendor dll.
  //   We need to check this.
  //   There are methods in webpackerator for this.

  ////////////////////

  let json

  if (useVendorDll) {

    let envOpts

    envOpts = {compileVendorDll: true, vendorChunkOrDll: 'dll', env}
    const vendorDllFileName = getAssetManifestFilename(envOpts)
    const vendorJson = getAssetManifestVendorDllJson(
      {fileName: vendorDllFileName, env, buildPath, envOpts})

    envOpts = {compileVendorDll: false, vendorChunkOrDll: 'dll', env}
    const otherFileName = getAssetManifestFilename(envOpts)
    const otherJson = getAssetManifestJson(
      {fileName: otherFileName, env, buildPath, envOpts})
    // TODO(vjpr): Do we need to pass something to handle merging arrays?
    //   What about nesting?

    json = _.merge(vendorJson, otherJson)

  } else {

    const envOpts = {compileVendorDll: false, vendorChunkOrDll: 'chunk', env}
    const fileName = getAssetManifestFilename(envOpts)

    json = getAssetManifestJson({fileName, env, buildPath, envOpts})

  }

  return json

}

////////////////////////////////////////////////////////////////////////////////

// TODO(vjpr): Should be shared with `live-frontend`.
function getAssetManifestFilename({compileVendorDll, vendorChunkOrDll, env}) {
  let filename = `webpack-assets`
  if (compileVendorDll) {
    filename += '.vendor-dll'
  } else {
    filename += `.vendor-as-${vendorChunkOrDll}`
  }
  filename += `.${env}.json`
  return filename
}

function getAssetManifestJson({fileName, env, buildPath, envOpts}) {
  const file = join(buildPath, fileName)
  const missingMessage = dedent`
    Asset manifest file not found.
      Env: ${c.dim(JSON.stringify(envOpts))}
      Missing file: ${c.dim(file)}
      You must run webpack (ensure correct environment and webpackerator config).
      Run ${c.dim(`NODE_ENV=${env} gulp webpack:dev-server`)}
      You must run your Node.js server with the correct USE_VENDOR_DLL env var.
  `
  assertExists({file, missingMessage})
  const json = fse.readJsonSync(file)
  return json
}

function getAssetManifestVendorDllJson({fileName, env, envOpts, buildPath, buildDir, noProxy, buildNoProxyPath}) {
  const file = join(buildPath, fileName)
  const missingMessage = dedent`
      Asset manifest file not found.
        Env: ${c.dim(JSON.stringify(envOpts))}
        Missing file: ${c.dim(file)}
        You must build the webpack vendor dll (ensure correct environment and webpackerator config).
        Run ${c.dim(`NODE_ENV=${env} MAKE_DLL=1 gulp webpack:build`)}
  `
  assertExists({file, missingMessage})
  let json = fse.readJsonSync(file)
  if (noProxy) {
    json.vendor.js = json.vendor.js.replace(buildDir, buildNoProxyPath)
  }
  return json
}

function assertExists({file, missingMessage}) {
  if (fse.existsSync(file)) return
  error(missingMessage)
  throw new Error(missingMessage)
}
