import React from 'react'
import ReactDOMServer from 'react-dom/server'
import path, {join} from 'path'
import {Html} from './html'
//import {serverRendering} from './server-rendering'

// TODO(vjpr): How to make this hot-reloadable for dev?
export default async function({isDev, publicDir, title, routes, store, webpackAssets, useServerSideRendering, loadingComponent}) {

  const js = getJs({isDev, webpackAssets, publicDir})
  const css = getCss({isDev, webpackAssets, publicDir})

  if (!useServerSideRendering) {
    const props = {
      js,
      css,
      store,
      title,
      loadingComponent,
    }
    const el =
      <Html {...props}
        component={<div/>}
      />
    return '<!doctype html>\n' + ReactDOMServer.renderToString(el)
  } else {
    const msg = 'Server-side rendering not implemented yet.'
    console.warn(msg)
    return msg
  }

}

function getJs({isDev, webpackAssets, publicDir}) {

  const assets = getAssets({publicDir})

  const externals = [
    // TODO(vjpr): This was hack during extraction.
    //assets.leafletJs.local,
  ]

  const bundles = [
    webpackAssets.vendor && webpackAssets.vendor.js,
    webpackAssets.main.js,
  ]

  const out = externals.concat(bundles).filter(Boolean)

  return out

}

function getCss({webpackAssets, publicDir}) {

  const assets = getAssets({publicDir})

  const externals = [
    // NOTE: Originally this came before main bundle css. The more you know.
    // TODO(vjpr): This was hack during extraction.
    //assets.leafletCss.local,
  ]

  // Will be undefined in dev/debug.
  const bundles = [
    webpackAssets.vendor && webpackAssets.vendor.css,
    webpackAssets.main.css,
  ]

  const out = externals.concat(bundles).filter(Boolean)

  return out

}

// TODO(vjpr): This is intranav specific. Remove it.
function getAssets({publicDir}) {
  const cdn = '//cdnjs.cloudflare.com/ajax/libs/'
  return {
    leafletCss: {
      local: join(publicDir, '/leaflet.css'),
    },
    leafletJs: {
      local: join(publicDir, '/leaflet-src.js'),
      cdn: 'http://cdn.leafletjs.com/leaflet-0.7.3/leaflet-src.js',
    },
    bootstrapCss: {
      local: join(publicDir, '/bootstrap.css'),
      cdn: cdn + 'twitter-bootstrap/3.3.5/css/bootstrap.css',
    },
    fontAwesomeCss: {
      local: join(publicDir, '/font-awesome.min.css'),
      cdn: cdn + 'font-awesome/4.3.0/css/font-awesome.min.css',
    },
  }
}

